# SPDX-License-Identifier: Apache-2.0

import wataash_utils.json
import wataash_utils.logger
try:
    import wataash_utils.requests
except ImportError:  # requests
    pass
import wataash_utils.pycharm
import wataash_utils.str

# -----------------------------------------------------------------------------
# misc

from wataash_utils.logger import logger


def warn_cr(file: str, txt: str) -> None:
    pos_cr = txt.find('\r')
    if pos_cr != -1:
        msg = (f'file {file}: '
               f'carriage-return (CR; 0x0d) character found at byte:{pos_cr} '
               f'(made on Windows?); '
               f'only line-feed (LF; 0x0a) is supported as the new-line '
               f'character')
        logger.warning(msg)


del logger



# SPDX-License-Identifier: Apache-2.0

import json
import typing as t

from wataash_utils.json import JSON
from wataash_utils.logger import logger
from wataash_utils.str import partial_str
import requests


def _debug_json_partially(jsn: JSON):
    logger.debug(partial_str(json.dumps(jsn, ensure_ascii=False), 50))


def _raise_for_status(resp: requests.Response, url: str, method: str):
    if 400 <= resp.status_code < 500 or 500 <= resp.status_code < 600:
        logger.error(f'{method} {url}: received {resp.status_code}')
        logger.error(f'resp.text: {resp.text}')
        resp.raise_for_status()


def json_get(*, url: str, params=None, allow_404=False, **kwargs) \
        -> t.Optional[JSON]:
    """
    :return: None on allow_404 && 404
    """

    logger.debug(f'GET {url}')
    resp = requests.get(url, params, **kwargs)
    if allow_404 and resp.status_code == 404:
        logger.debug(f'404: {resp.text}')
        return None
    _raise_for_status(resp, url, 'GET')
    logger.debug(f'{resp.status_code} {resp.reason}')
    r_json = resp.json()
    _debug_json_partially(r_json)
    return r_json


def json_post(*, url: str, json: JSON, **kwargs) -> JSON:
    logger.debug(f'POST {url} {partial_str(str(json))}')
    resp = requests.post(url, json=json, **kwargs)
    _raise_for_status(resp, url, 'POST')
    logger.debug(f'{resp.status_code} {resp.reason}')
    r_json = resp.json()
    _debug_json_partially(r_json)
    return r_json


def json_patch(*, url: str, json: JSON, **kwargs) -> JSON:
    logger.debug(f'PATCH {url} {partial_str(str(json))}')
    resp = requests.patch(url, json=json, **kwargs)
    _raise_for_status(resp, url, 'PATCH')
    logger.debug(f'{resp.status_code} {resp.reason}')
    r_json = resp.json()
    _debug_json_partially(r_json)
    return r_json


def json_delete(*, url: str, json: JSON, **kwargs) -> JSON:
    logger.debug(f'DELETE {url} {partial_str(str(json))}')
    resp = requests.delete(url, json=json, **kwargs)
    _raise_for_status(resp, url, 'DELETE')
    logger.debug(f'{resp.status_code} {resp.reason}')
    r_json = resp.json()
    _debug_json_partially(r_json)
    return r_json



# SPDX-License-Identifier: Apache-2.0

import typing as t


def cat_and(strings: t.List[str]) -> str:
    """
    >>> cat_and([])
    ''
    >>> cat_and(['a'])
    'a'
    >>> cat_and(['a', 'b'])
    'a and b'
    >>> cat_and(['a', 'b', 'c'])
    'a, b, and c'
    >>> cat_and(['a', 'b', 'c', 'd'])
    'a, b, c, and d'
    """
    if len(strings) == 0:
        return ''
    if len(strings) == 1:
        return strings[0]
    if len(strings) == 2:
        return f'{strings[0]} and {strings[1]}'
    return ', '.join(strings[:-1]) + f', and {strings[-1]}'


def cat_or(strings: t.List[str]) -> str:
    """
    >>> cat_or([])
    ''
    >>> cat_or(['a'])
    'a'
    >>> cat_or(['a', 'b'])
    'a or b'
    >>> cat_or(['a', 'b', 'c'])
    'a, b, or c'
    >>> cat_or(['a', 'b', 'c', 'd'])
    'a, b, c, or d'
    """
    if len(strings) == 0:
        return ''
    if len(strings) == 1:
        return strings[0]
    if len(strings) == 2:
        return f'{strings[0]} or {strings[1]}'
    return ', '.join(strings[:-1]) + f', or {strings[-1]}'


def partial_str(s: str, width=100) -> str:
    """
    >>> partial_str('abcd', 5)
    'abcd'

    >>> partial_str('abcde', 5)
    'abcde'

    >>> partial_str('abcdef', 5)
    'ab...'

    >>> partial_str('abcd', 2)
    Traceback (most recent call last):
        ...
    ValueError: width must be greater than 3
    """

    if width <= 3:
        raise ValueError('width must be greater than 3')

    if len(s) > width:
        s = s[:width - len('...')] + '...'
    return s



# SPDX-License-Identifier: Apache-2.0

import logging
import os

if 'PYCHARM_HOSTED' in os.environ:
    # must before import logzero
    os.environ['LOGZERO_FORCE_COLOR'] = '1'

import logzero  # pylint: disable=wrong-import-order,wrong-import-position

# difference with logzero.DEFAULT_FORMAT: %(message)s%(end_color)s
# difference with logzero.DEFAULT_COLORS:
# - logging.DEBUG: ForegroundColors.CYAN,
# - logging.INFO: ForegroundColors.GREEN,
_ = logzero.DEFAULT_FORMAT
_ = logzero.DEFAULT_COLORS
logzero.formatter(logzero.LogFormatter(
    fmt=('%(color)s[%(levelname)1.1s %(asctime)s '
         '%(filename)s:%(lineno)d %(funcName)s()] %(message)s%(end_color)s'),
    colors={
        logging.DEBUG: logzero.ForegroundColors.WHITE,
        logging.INFO: logzero.ForegroundColors.BLUE,
        logging.WARNING: logzero.ForegroundColors.YELLOW,
        logging.ERROR: logzero.ForegroundColors.RED,
        logging.CRITICAL: logzero.ForegroundColors.RED
    }
))

logger: logging.Logger = logzero.logger  # pylint: disable=invalid-name



# SPDX-License-Identifier: Apache-2.0

# https://click.palletsprojects.com/en/7.x/setuptools/

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='wataash_utils',
    version='0.1.3',
    url='https://github.com/wataash/wataash_utils_py',
    author='Wataru Ashihara',
    author_email='wataash@wataash.com',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    license='Apache-2.0',
    description='wataash\'s personal utilities',
    long_description=long_description,
    keywords='wataash_utils',
    install_requires=[
        'logzero>=1.6.3',
    ],
    packages=find_packages(),
)



# SPDX-License-Identifier: Apache-2.0

import logging

import click


def parse_quiet_verbose(ctx: click.Context, quiet: bool, verbose: int) -> int:
    if quiet and verbose:
        msg = '--quiet and --verbose are mutually exclusive'
        raise click.UsageError(msg, ctx=ctx)
    if quiet:
        return logging.ERROR
    elif verbose == 0:
        return logging.WARNING
    elif verbose == 1:
        return logging.INFO
    elif verbose >= 2:
        return logging.DEBUG



# SPDX-License-Identifier: Apache-2.0

import os


def connect(port=12345):
    import pydevd_pycharm

    pydevd_pycharm.settrace('localhost', port=port,
                            stdoutToServer=True, stderrToServer=True,
                            suspend=False)


def connect_if_env(port=12345, env='CONNECT_PYCHARM'):
    if os.getenv(env) != '1':
        return

    connect()
